'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.paginationFolderSuffix = paginationFolderSuffix;
exports.getPaginationListParams = getPaginationListParams;
exports.paginateQuery = paginateQuery;
exports.createMapStateToPropsPagination = createMapStateToPropsPagination;
exports['default'] = paginate;

var _actions = require('../actions');

var _containersList = require('./containers/list');

var _containersCommon = require('./containers/common');

var defaultOpts = {
  rowsPerPage: 10
};

exports.defaultOpts = defaultOpts;

function paginationFolderSuffix(rowsPerPage, startkey) {
  return '/page/' + rowsPerPage + '/' + (startkey || '');
}

// add limit, startkey to list params

function getPaginationListParams(listParams, opts, rowsPerPage, startkey) {
  var finalStartkey = startkey || opts.options && opts.options.startkey;
  return Object.assign({}, listParams, { limit: rowsPerPage + 1 }, startkey ? { startkey: finalStartkey } : null);
}

function paginateQuery(crud, opts, rowsPerPage, startkey) {
  var fun = undefined;
  var listParams = (0, _containersList.getListParams)(crud, opts);
  var funParams = getPaginationListParams(listParams, opts, rowsPerPage, startkey);
  if ((0, _containersList.isQuery)(opts)) {
    fun = crud.db.query.bind(crud.db, opts.options.fun);
  } else {
    fun = crud.db.allDocs.bind(crud.db);
  }

  return fun(funParams).then(function (payload) {
    //assign next page starting id
    if (payload.rows[rowsPerPage]) {
      var lastRow = payload.rows.pop();
      payload.next = lastRow.id;
    }
    // create reversedParams from funParams
    var reversedParams = _extends({}, funParams, {
      startkey: payload.rows[0].id,
      endkey: listParams.startkey,
      limit: rowsPerPage,
      skip: 1,
      descending: true
    });
    return fun(reversedParams).then(function (r) {
      var firstRow = r.rows.pop();
      if (firstRow) {
        payload.prev = firstRow.id;
      }
      return payload;
    });
  });
}

function createMapStateToPropsPagination(paginationOpts, crud, opts, mapStateToProps) {
  if (paginationOpts === undefined) paginationOpts = {};
  if (opts === undefined) opts = {};

  return function (state, ownProps) {
    var props = mapStateToProps ? mapStateToProps(state, ownProps) : {};
    // rowsPerPage and startkey can be given in mapStateToProps,
    // paginationOpts or defaultOpts

    var _Object$assign = Object.assign(defaultOpts, paginationOpts, props);

    var rowsPerPage = _Object$assign.rowsPerPage;
    var startkey = _Object$assign.startkey;

    // finalOpts from argument opts or from mapStateToProps
    var finalOpts = Object.assign(opts, props.listOpts);
    // set pouchdb options with pagination related things
    finalOpts.options = Object.assign({}, finalOpts.options, {
      limit: rowsPerPage + 1,
      startkey: startkey
    });
    var toFolder = (finalOpts.folder || (0, _containersList.folderNameFromOpts)(finalOpts.options)) + paginationFolderSuffix(rowsPerPage, startkey);
    var propName = finalOpts.propName || 'items';
    // add documents, folderVars to props
    Object.assign(props, (0, _containersList.createMapStateToProps)(crud.mountPoint, toFolder, propName)(state));
    // assign action that loads items from db
    props.action = function () {
      return (0, _actions.createPromiseAction)(function () {
        return paginateQuery(crud, toFolder, rowsPerPage, startkey);
      }, crud.actionTypes.query, { folder: toFolder });
    };
    return props;
  };
}

function paginate(paginationOpts, crud, connectListOpts, mapStateToProps, mapDispatchToProps) {
  var mapStateToPropsFinal = createMapStateToPropsPagination(paginationOpts, crud, connectListOpts, mapStateToProps);
  return (0, _containersCommon.wrap)(mapStateToPropsFinal, mapDispatchToProps);
}